<?php

declare(strict_types=1);

namespace Dedi\SyliusStockHistoryPlugin\EventListener;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\SyliusStockHistoryPlugin\DependencyInjection\Settings\SettingsInterface;
use Dedi\SyliusStockHistoryPlugin\Factory\InventoryLogEntryFactoryInterface;
use Dedi\SyliusStockHistoryPlugin\Resolver\InventoryLogEntryContextResolverInterface;
use Dedi\SyliusStockHistoryPlugin\Resolver\Order\InventoryLogOrderResolverInterface;
use Doctrine\ORM\Event\OnFlushEventArgs;
use Sylius\Component\Core\Model\ProductVariantInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class OnFlushInventoryListener
{
    public function __construct(
        private readonly SettingsProviderInterface $settingsProvider,
        private readonly InventoryLogEntryFactoryInterface $inventoryLogEntryFactory,
        private readonly InventoryLogEntryContextResolverInterface $inventoryLogEntryContextResolver,
        private readonly InventoryLogOrderResolverInterface $inventoryLogOrderResolver,
        private readonly RequestStack $requestStack,
    ) {
    }

    public function onFlush(OnFlushEventArgs $eventArgs): void
    {
        $entityManager = $eventArgs->getObjectManager();
        $unitOfWork = $entityManager->getUnitOfWork();

        $scheduledEntities = array_merge(
            $unitOfWork->getScheduledEntityInsertions(),
            $unitOfWork->getScheduledEntityUpdates(),
        );

        foreach ($scheduledEntities as $entity) {
            if (!$entity instanceof ProductVariantInterface) {
                continue;
            }

            /** @var SettingsInterface $settings */
            $settings = $this->settingsProvider->provide();
            if (!$settings->isEnabled()) {
                return;
            }

            $changeSet = $unitOfWork->getEntityChangeSet($entity);

            if (!array_key_exists('onHand', $changeSet) &&
                !array_key_exists('onHold', $changeSet)
            ) {
                continue;
            }

            $previousOnHand = array_key_exists('onHand', $changeSet) ? $changeSet['onHand'][0] : null;
            $previousOnHold = array_key_exists('onHold', $changeSet) ? $changeSet['onHold'][0] : null;

            if (null === $previousOnHold && null === $previousOnHand) {
                continue;
            }

            $onHand = array_key_exists('onHand', $changeSet) ? $changeSet['onHand'][1] : $entity->getOnHand();
            $onHold = array_key_exists('onHold', $changeSet) ? $changeSet['onHold'][1] : $entity->getOnHold();

            $request = $this->requestStack->getCurrentRequest();
            $path = null === $request ? '' : $request->getPathInfo();

            $context = $this->inventoryLogEntryContextResolver->resolve();
            $contextType = $context::getType();

            $order = $this->inventoryLogOrderResolver->supports($contextType) ? $this->inventoryLogOrderResolver->resolve($contextType) : null;

            $inventoryLogEntry = $this->inventoryLogEntryFactory->create(
                $entity,
                $contextType,
                $onHand,
                $onHold,
                $previousOnHand,
                $previousOnHold,
                $path,
                $order,
            );

            $entityManager->persist($inventoryLogEntry);
            $unitOfWork->computeChangeSets();
        }
    }
}
